import string
import six

from ipalib import api, errors, output, _
from ipalib import Str, StrEnum, Bool, Int, _
from ipalib.plugable import Registry
from . import baseuser
from .baseuser import (
    baseuser,
    baseuser_add,
    baseuser_del,
    baseuser_mod,
    baseuser_find,
    baseuser_show,
    NO_UPG_MAGIC,
    UPG_DEFINITION_DN,
    baseuser_output_params,
    baseuser_pwdchars,
    validate_nsaccountlock,
    convert_nsaccountlock,
    fix_addressbook_permission_bindrule,
    baseuser_add_manager,
    baseuser_remove_manager,
    baseuser_add_principal,
    baseuser_remove_principal)
from . import baseldap
from .baseldap import (LDAPObject, LDAPCreate, LDAPDelete,
                                     LDAPUpdate, LDAPSearch, LDAPRetrieve,
                                     LDAPQuery, LDAPAddMember, LDAPRemoveMember,
                                     add_external_pre_callback,
                                     add_external_post_callback,
                                     remove_external_post_callback,
                                     output, entry_to_dict, pkey_to_value,
                                     LDAPAddAttribute, LDAPRemoveAttribute,
                                     LDAPModAttribute)
from .hbacrule import is_all
from ipalib import _, ngettext
from ipalib.util import validate_hostmask
from ipapython.dn import DN

if six.PY3:
    unicode = str


__doc__ = _("""
Parsec policy
""")

register = Registry()

topic = 'parsec'

@register()
class usermac(LDAPObject):
    """
    User MAC
    """
    container_dn = baseuser.active_container_dn
    object_name = _('usermac')
    object_name_plural = _('usermacs')
    object_class = ['top', 'nscontainer', 'x-ald-user-parsec14']
    default_attributes = ['x-ald-user-mac', 'xaldusermacmin', 'xaldusermacmax']
    search_attributes = ['x-ald-user-mac']

    label = _('User MACs')
    label_singular = _('User MAC')

    takes_params = (
        Str('xaldusermacmin',
            label=_('Min. MAC'),
        ),
        Str('xaldusermacmax',
            label=_('Max. MAC'),
        ),
        Str('x-ald-user-mac',
            cli_name='usermac',
            label=_('User MAC'),
            default_from=lambda xaldusermacmin, xaldusermacmax: xaldusermacmin + ':0x0:'  + xaldusermacmax + ':0x0',
            autofill=True,
        )
     )

    def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
        self.obj.get_container_attribute(entry_attrs, options)

        return dn

@register()
class usermac_add(LDAPCreate):
    __doc__ = _('Create new user MAC.')

    msg_summary = _('Added user MAC"%(value)s"')

@register()
class usermac_del(LDAPDelete):
    __doc__ = _('Delete user MAC.')

    msg_summary = _('Deleted user MAC"%(value)s"')

@register()
class usermac_mod(LDAPUpdate):
    __doc__ = _('Modify user MAC.')

    msg_summary = _('Modified user MAC"%(value)s"')

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)
        validate_domain_level(self.api)

        f1=open('/tmp/parsec', 'w+')
        f1.write('Change user MAC pre')
        f1.close()

        return dn

    def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
        self.obj.get_container_attribute(entry_attrs, options)

        f1=open('/tmp/parsec', 'w+')
        f1.write('Change user MAC post')
        f1.close()

        return dn


@register()
class usermac_find(LDAPSearch):
    __doc__ = _('Search for user MAC.')

    msg_summary = ngettext(
        '%(count)d user MAC matched',
        '%(count)d user MAC matched', 0
    )

@register()
class usermac_show(LDAPRetrieve):
    __doc__ = _('Display user MAC.')



@register()
class maclevels(baseldap.LDAPObject):
    """
    MAC Level
    """
    container_dn = DN(('cn', 'mac-levels'),
                      ('cn', 'mac-services'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('maclevel')
    object_name_plural = _('maclevels')
    object_class = ['top', 'nscontainer', 'x-ald-mac-level-object']
    default_attributes = ['cn', 'x-ald-mac-level']
    search_attributes = ['cn', 'x-ald-mac-level']


    label = _('MAC Levels')
    label_singular = _('MAC Level')

    takes_params = (
        Str('cn',
            cli_name='maclevel_name',
            label=_('MAC Level'),
            primary_key=True,
        ),
        Int('x-ald-mac-level',
            cli_name='maclevel',
            label=_('Levels of the MAC'),
            doc=_('integer to order the MAC leveles'),
            minvalue=0,
            maxvalue=255,
        ),
    )

@register()
class maclevels_add(LDAPCreate):
    __doc__ = _('Create new MAC Level.')

    msg_summary = _('Added MAC Level"%(value)s"')

@register()
class maclevels_del(LDAPDelete):
    __doc__ = _('Delete MAC Level.')

    msg_summary = _('Deleted MAC Level"%(value)s"')


@register()
class maclevels_mod(LDAPUpdate):
    __doc__ = _('Modify MAC Level.')

    msg_summary = _('Modified MAC Level"%(value)s"')




@register()
class maclevels_find(LDAPSearch):
    __doc__ = _('Search for MAC Level.')

    msg_summary = ngettext(
        '%(count)d MAC Level matched',
        '%(count)d MAC Level matched', 0
    )

@register()
class maclevels_show(LDAPRetrieve):
    __doc__ = _('Display MAC Level.')

"""
@register()
class maclevels_add_member(LDAPAddMember):
    __doc__ = _('Add members to MAC Group.')

@register()
class maclevels_remove_member(LDAPRemoveMember):
    __doc__ = _('Remove members from MAC Group.')

"""

@register()
class miclevels(baseldap.LDAPObject):
    """
    MIC Levels
    """
    container_dn = DN(('cn', 'mic-levels'),
                      ('cn', 'mic-services'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('miclevel')
    object_name_plural = _('miclevels')
    object_class = ['top', 'nscontainer', 'x-ald-user']
    default_attributes = ['cn', 'x-ald-user-mic-level']

    label = _('MIC Levels')
    label_singular = _('MIC Level')

    takes_params = (
        Str('cn',
            cli_name='miclevel_name',
            label=_('MIC Level'),
            primary_key=True,
        ),
        Int('x-ald-user-mic-level',
            cli_name='miclevel',
            label=_('Levels of the MIC'),
            doc=_('integer to order the MIC leveles'),
            minvalue=0,
            maxvalue=255,
        ),
    )


#    def get_dn(self, *keys, **kwargs):
#        if keys[-1] is not None:
#            return self.api.Object.user.get_dn(*keys, **kwargs)
#        return DN(self.container_dn, api.env.basedn)

@register()
class miclevels_add(LDAPCreate):
    __doc__ = _('Create new MIC level.')

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)
        self.obj.check_order_uniqueness(*keys, **options)
        return dn

    msg_summary = _('Added MIC Level "%(value)s"')

@register()
class miclevels_del(LDAPDelete):
    __doc__ = _('Delete MIC Level.')

    msg_summary = _('Deleted MIC Level "%(value)s"')

@register()
class miclevels_mod(LDAPUpdate):
    __doc__ = _('Modify MIC Level.')

    msg_summary = _('Modified MIC Level"%(value)s"')

@register()
class miclevels_find(LDAPSearch):
    __doc__ = _('Search for MIC Level.')

    msg_summary = ngettext(
        '%(count)d MIC Level matched',
        '%(count)d MIC Level matched', 0
    )

@register()
class miclevels_show(LDAPRetrieve):
    __doc__ = _('Display MIC Level.')